//----------------------------------------------------------------------
//               The Motion Strategy Library (MSL)
//----------------------------------------------------------------------
//
// Copyright (c) 1998-2000 Iowa State University and Steve LaValle.  
// All Rights Reserved.
// 
// Permission to use, copy, and distribute this software and its 
// documentation is hereby granted free of charge, provided that 
// (1) it is not a component of a commercial product, and 
// (2) this notice appears in all copies of the software and
//     related documentation. 
// 
// Iowa State University and the author make no representations
// about the suitability or fitness of this software for any purpose.  
// It is provided "as is" without express or implied warranty.
//----------------------------------------------------------------------

#ifndef MSL_3DTRIANGLE_H
#define MSL_3DTRIANGLE_H

#include <LEDA/d3_point.h> 
#include <LEDA/polygon.h> 
#include <LEDA/segment.h> 

#include <LEDA/REDEFINE_NAMES.h>

//! A 3D triangle, made of 3 LEDA 3D points

class Triangle{
 public:
  d3_point p1,p2,p3;

  Triangle(d3_point pt1, d3_point pt2, d3_point pt3);
  Triangle();
  ~Triangle(){}
  Triangle(const Triangle& tr);
  friend istream& operator>> (istream& is, Triangle& tr) 
    { is >> tr.p1 >> tr.p2 >> tr.p3; return is; }
  friend ostream& operator<< (ostream& os, const Triangle& tr)
    { os << tr.p1 << " " << tr.p2 << " " << tr.p3 << endl; return os; }
  const Triangle& operator= (const Triangle& tr)
   { p1=tr.p1; p2=tr.p2; p3=tr.p3; return (*this);}
};


//! A handy utility for converting 2D geometries into 3D
list<Triangle> PolygonsToTriangles(const list<polygon> &pl,
				   double thickness);

//! A handy utility for projecting 3D geometries into 2D
list<polygon> TrianglesToPolygons(const list<Triangle> &tl);

#include <LEDA/UNDEFINE_NAMES.h>

#endif
