//----------------------------------------------------------------------
//               The Motion Strategy Library (MSL)
//----------------------------------------------------------------------
//
// Copyright (c) 1998-2000 Iowa State University and Steve LaValle.  
// All Rights Reserved.
// 
// Permission to use, copy, and distribute this software and its 
// documentation is hereby granted free of charge, provided that 
// (1) it is not a component of a commercial product, and 
// (2) this notice appears in all copies of the software and
//     related documentation. 
// 
// Iowa State University and the author make no representations
// about the suitability or fitness of this software for any purpose.  
// It is provided "as is" without express or implied warranty.
//----------------------------------------------------------------------

#include "3Dtriangle.h"

#include <LEDA/REDEFINE_NAMES.h>

Triangle::Triangle(d3_point pt1, d3_point pt2, d3_point pt3){
  p1 = pt1;
  p2 = pt2;
  p3 = pt3;
}


Triangle::Triangle(){
  d3_point *p;
  p = new d3_point(0,0,0);
  p1 = *p;
  p2 = *p;
  p3 = *p;
}


Triangle::Triangle(const Triangle& p){
  p1 = p.p1;
  p2 = p.p2;
  p3 = p.p3;
}


list<Triangle> PolygonsToTriangles(const list<polygon> &pl,
				   double thickness) {
  polygon p;
  segment s;
  point pt;
  d3_point p1,p2,p3;
  list<Triangle> tl;

  tl.clear();

  forall(p,pl) {
    // Make the sides
    forall_segments(s,p) {
      p1 = d3_point(s.xcoord1(),s.ycoord1(),0.0);
      p2 = d3_point(s.xcoord2(),s.ycoord2(),0.0);
      p3 = d3_point(s.xcoord2(),s.ycoord2(),thickness);
      tl.push(Triangle(p1,p2,p3));
    }
    forall_segments(s,p) {
      p1 = d3_point(s.xcoord1(),s.ycoord1(),thickness);
      p2 = d3_point(s.xcoord2(),s.ycoord2(),thickness);
      p3 = d3_point(s.xcoord1(),s.ycoord1(),0.0);
      tl.push(Triangle(p1,p3,p2));
      pt = s.start(); // Get a point for later
    }
    // Make the top
    forall_segments(s,p) {
      if ((s.start() != pt)&&(s.end() != pt)) {
	p1 = d3_point(s.xcoord1(),s.ycoord1(),thickness);
	p2 = d3_point(s.xcoord2(),s.ycoord2(),thickness);
	p3 = d3_point(pt.xcoord(),pt.ycoord(),thickness);
	tl.push(Triangle(p1,p2,p3));
      }
    }
   }

  return tl;
}


list<polygon> TrianglesToPolygons(const list<Triangle> &tl) {
  Triangle tr;
  polygon p;
  d3_point p1,p2,p3;
  list<polygon> pl;
  list<point> ptl;

  pl.clear();

  forall(tr,tl) {
    ptl.clear();
    ptl.push(point(tr.p1.xcoord(),tr.p1.ycoord()));
    ptl.push(point(tr.p2.xcoord(),tr.p2.ycoord()));
    ptl.push(point(tr.p3.xcoord(),tr.p3.ycoord()));
    p = polygon(ptl,polygon::NO_CHECK);
    pl.push(p);
  }

  return pl;
}


#include <LEDA/UNDEFINE_NAMES.h>





