//----------------------------------------------------------------------
//               The Motion Strategy Library (MSL)
//----------------------------------------------------------------------
//
// Copyright (c) 1998-2000 Iowa State University and Steve LaValle.  
// All Rights Reserved.
// 
// Permission to use, copy, and distribute this software and its 
// documentation is hereby granted free of charge, provided that 
// (1) it is not a component of a commercial product, and 
// (2) this notice appears in all copies of the software and
//     related documentation. 
// 
// Iowa State University and the author make no representations
// about the suitability or fitness of this software for any purpose.  
// It is provided "as is" without express or implied warranty.
//----------------------------------------------------------------------

#include <math.h>
#include <stdio.h>
#include <fstream.h>

#include "gui.h"
#include "defs.h"

#include <LEDA/REDEFINE_NAMES.h>


//////////////////////////////////////////////////////////////////////////
//
//  Gui class
//
//////////////////////////////////////////////////////////////////////////

Gui::Gui(Render *render) {
  R = render;

  FilePath = R->FilePath;
}


void Gui::Init() 
{
  // Perform Render initialization
  R->Init();
}



void Gui::Start() 
{
  // Perform initialization
  Init();

  // Enter the event processing loop
  MainLoop();
}



void Gui::HandleEvents() 
{
  double x,y;
  int val,e;
  window *wp = NULL;

  e = get_event(wp,val,x,y);
  
  if (e == button_press_event) {
    // The Gui window buttons
    if (wp == W) 
      ButtonHandle(val);
    // The Render control window buttons
    if ((R->RenderCtlWindowOn)&&(wp == R->RenderCtlWindow)) {
      R->ButtonHandle(val);
      R->RenderCtlWindow->redraw_panel();
    }
  }
}



void Gui::MainLoop() 
{
  if (R->ControlFreak) { // Does the renderer NEED to be in control?
    R->MainLoop(this); // Give a pointer to this Gui
    // It is the responsibility of Render to handle Gui events
  }
  else {
    Finished = false;
    while (!Finished) {
      R->HandleEvents(); // Handle events in the renderer
      HandleEvents();
    }
  }

  R->Terminate();  // If we ever get here (some control freaks don't allow it)
}


#include <LEDA/UNDEFINE_NAMES.h>

