//----------------------------------------------------------------------
//               The Motion Strategy Library (MSL)
//----------------------------------------------------------------------
//
// Copyright (c) 1998-2000 Iowa State University and Steve LaValle.  
// All Rights Reserved.
// 
// Permission to use, copy, and distribute this software and its 
// documentation is hereby granted free of charge, provided that 
// (1) it is not a component of a commercial product, and 
// (2) this notice appears in all copies of the software and
//     related documentation. 
// 
// Iowa State University and the author make no representations
// about the suitability or fitness of this software for any purpose.  
// It is provided "as is" without express or implied warranty.
//----------------------------------------------------------------------

#ifndef MSL_GUI_H
#define MSL_GUI_H

#include <LEDA/list.h>
#include <LEDA/matrix.h>
#include <LEDA/point.h>
#include <LEDA/polygon.h>
#include <LEDA/stream.h>
#include <LEDA/string.h>
#include <LEDA/vector.h>
#include <LEDA/window.h>
#include <LEDA/menu.h>

#include "problem.h"
#include "scene.h"
#include "render.h"

class Render;  // This needs to be here because of mutual reference

#include <LEDA/REDEFINE_NAMES.h>

//! A generic class for designing graphical user interfaces (GUIs)
/*!
The graphical user interface (GUI) is designed as a hierarchy of
classes to enable specific user interfaces to be designed for a
variety of different motion strategy problems and planning algorithms.
Currently, there is one derived class which serves as the GUI for all
of the RRT-based planners.  Each instance of Gui includes an instance
of an RRT Planner class and an instance of a Render class.  Using this
design, the same basic GUI design can be used, regardless of the
particular rendering methods.  
*/

class Gui {
 protected:
  window *W;
  Render *R;
  string FilePath;

  //! Make the menu window
  virtual void CreateMenuWindow() {};

  //! Figure out what actions to take based on menu choices
  virtual void ButtonHandle(int b) {};

  //! Initialize Gui and Render
  virtual void Init();

  //! The main event processing loop
  virtual void MainLoop();

 public:
  Gui(Render *render);
  virtual ~Gui() {};

  //! Start running the Gui
  virtual void Start();

  //! Set to true if you want to main loop to terminate
  bool Finished;

  //! Process any IO events (may be used by Render)
  virtual void HandleEvents();

};

#include <LEDA/UNDEFINE_NAMES.h>

#endif
