//----------------------------------------------------------------------
//               The Motion Strategy Library (MSL)
//----------------------------------------------------------------------
//
// Copyright (c) 1998-2000 Iowa State University and Steve LaValle.  
// All Rights Reserved.
// 
// Permission to use, copy, and distribute this software and its 
// documentation is hereby granted free of charge, provided that 
// (1) it is not a component of a commercial product, and 
// (2) this notice appears in all copies of the software and
//     related documentation. 
// 
// Iowa State University and the author make no representations
// about the suitability or fitness of this software for any purpose.  
// It is provided "as is" without express or implied warranty.
//----------------------------------------------------------------------

#ifndef MSL_GUIPLANNER_H
#define MSL_GUIPLANNER_H

#include <LEDA/file.h>
#include <LEDA/array.h>
#include <LEDA/list.h>
#include <LEDA/point.h>
#include <LEDA/polygon.h>
#include <LEDA/stream.h>
#include <LEDA/string.h>
#include <LEDA/vector.h>
#include <LEDA/graph.h>
#include <LEDA/window.h>
#include <LEDA/menu.h>
#include <LEDA/ps_file.h>

#include "gui.h"
#include "planner.h"
#include "rrt.h"
#include "prm.h"
#include "fdp.h"

#include <LEDA/REDEFINE_NAMES.h>

//! A rendering-independent GUI for the Planner classes

class GuiPlanner: public Gui {
 protected:
  virtual void Init();
  virtual void CreateMenuWindow();
  virtual void ButtonHandle(int b);
  window *PlanCtlWindow;
  bool PlanCtlWindowOn;
 public:
  double LineWidth;
  double PSLineWidth;
  int DrawIndexX,DrawIndexY;
  Planner *Pl;
  GuiPlanner(Render *render, Planner *planner);
  virtual ~GuiPlanner(){};
  void ResetPlanner();
  void WriteGraphs();
  void ReadGraphs();
  void DrawGraphs();
  void ReadAnimationFrames();
  void WriteAnimationFrames();
  void ReadPath();
  void WritePath();
  void PSDrawGraphs();
};

#include <LEDA/UNDEFINE_NAMES.h>

#endif
