//----------------------------------------------------------------------
//               The Motion Strategy Library (MSL)
//----------------------------------------------------------------------
//
// Copyright (c) 1998-2000 Iowa State University and Steve LaValle.  
// All Rights Reserved.
// 
// Permission to use, copy, and distribute this software and its 
// documentation is hereby granted free of charge, provided that 
// (1) it is not a component of a commercial product, and 
// (2) this notice appears in all copies of the software and
//     related documentation. 
// 
// Iowa State University and the author make no representations
// about the suitability or fitness of this software for any purpose.  
// It is provided "as is" without express or implied warranty.
//----------------------------------------------------------------------

#include "marray.h"

#include <LEDA/REDEFINE_NAMES.h>


template<class E> MultiArray<E>::MultiArray(const array<int> &dims, 
					    const E &x) {
  int i,offset;

  MaxSize = 10000000; // Maximum allowable size
  Dimensions = dims;
  Dimension = dims.size();
  
  Offsets = array<int>(Dimension);

  offset = 1;
  for (i = 0; i < Dimension; i++) {
    Offsets[i] = offset;
    offset *= Dimensions[i];
  }

  Size = offset;

  if (Size <= MaxSize) {
    // Make the LEDA array to hold all of the data
    A = array<E>(Size);
    A.init(x); // Write the value x to all elements
  }
  else {
    cout << "Size " << Size << " exceeds MaxSize limit " << MaxSize << "\n";
    exit(-1);
  }
}


template<class E> MultiArray<E>::MultiArray(const array<int> &dims) {
  E x;

  MultiArray(dims,x);
}


template<class E> inline E& MultiArray<E>::operator[](const array<int> 
						      &indices) {
  int i,index;

  index = indices[0];

  for (i = 1; i < Dimension; i++) {
    index += indices[i]*Offsets[i];
  }

  return A[index];
}

#include <LEDA/UNDEFINE_NAMES.h>

