//----------------------------------------------------------------------
//               The Motion Strategy Library (MSL)
//----------------------------------------------------------------------
//
// Copyright (c) 1998-2000 Iowa State University and Steve LaValle.  
// All Rights Reserved.
// 
// Permission to use, copy, and distribute this software and its 
// documentation is hereby granted free of charge, provided that 
// (1) it is not a component of a commercial product, and 
// (2) this notice appears in all copies of the software and
//     related documentation. 
// 
// Iowa State University and the author make no representations
// about the suitability or fitness of this software for any purpose.  
// It is provided "as is" without express or implied warranty.
//----------------------------------------------------------------------

#ifndef MSL_MODELMISC_H
#define MSL_MODELMISC_H

#include <LEDA/file.h>
#include <LEDA/list.h>
#include <LEDA/matrix.h>
#include <LEDA/point.h>
#include <LEDA/polygon.h>
#include <LEDA/random.h>
#include <LEDA/stream.h>
#include <LEDA/string.h>
#include <LEDA/vector.h>

#include "model.h"

#include <LEDA/REDEFINE_NAMES.h>


//! A simple one-dimensional model for dynamics studies
class Model1D: public Model {
 public:
  double Force;
  Model1D(string path);
  virtual ~Model1D() {};
  virtual point StateToLedaPoint(const vector &x);
  virtual vector StateToConfiguration(const vector &x);
  virtual vector Integrate(const vector &x, const vector &u, const double &h);
  virtual vector StateTransitionEquation(const vector &x, const vector &u);
  virtual double Metric(const vector &x1, const vector &x2);
};

//! A linear systems model:   xdot = Ax + Bu
class ModelLinear: public Model {
 public:
  matrix A;
  matrix B;
  ModelLinear(string path);
  virtual ~ModelLinear() {};
  virtual point StateToLedaPoint(const vector &x);
  virtual vector StateToConfiguration(const vector &x);
  virtual vector Integrate(const vector &x, const vector &u, const double &h);
  virtual vector StateTransitionEquation(const vector &x, const vector &u);
};


//! Simple axis-parallel motions in an N-dimensional space 
class ModelND: public Model {
 public:
  double CorridorWidth;
  ModelND(string path);
  virtual ~ModelND() {};
  virtual point StateToLedaPoint(const vector &x);
  virtual vector StateToConfiguration(const vector &x);
  virtual vector Integrate(const vector &x, const vector &u, const double &h);
  virtual vector StateTransitionEquation(const vector &x, const vector &u);
  //virtual bool Satisfied(const vector &x);
};


//! The "nonholonomic integrator", used by R. Brockett and many others.
class ModelNintegrator: public Model {
 public:
  double UBound;
  double VBound;
  ModelNintegrator(string path);
  virtual ~ModelNintegrator() {};
  virtual point StateToLedaPoint(const vector &x);
  virtual vector StateToConfiguration(const vector &x);
  virtual vector Integrate(const vector &x, const vector &u, const double &h);
  virtual vector StateTransitionEquation(const vector &x, const vector &u);
};


#include <LEDA/UNDEFINE_NAMES.h>

#endif
