//----------------------------------------------------------------------
//               The Motion Strategy Library (MSL)
//----------------------------------------------------------------------
//
// Copyright (c) 1998-2000 Iowa State University and Steve LaValle.  
// All Rights Reserved.
// 
// Permission to use, copy, and distribute this software and its 
// documentation is hereby granted free of charge, provided that 
// (1) it is not a component of a commercial product, and 
// (2) this notice appears in all copies of the software and
//     related documentation. 
// 
// Iowa State University and the author make no representations
// about the suitability or fitness of this software for any purpose.  
// It is provided "as is" without express or implied warranty.
//----------------------------------------------------------------------

#ifndef MSL_RENDERLEDA_H
#define MSL_RENDERLEDA_H

#include <LEDA/file.h>
#include <LEDA/list.h>
#include <LEDA/matrix.h>
#include <LEDA/point.h>
#include <LEDA/polygon.h>
#include <LEDA/stream.h>
#include <LEDA/string.h>
#include <LEDA/vector.h>
#include <LEDA/d3_point.h>
#include <LEDA/window.h>

#include "3Dtriangle.h"
#include "render.h"

#include <LEDA/REDEFINE_NAMES.h>

//! Perform 2D rendering using LEDA
class RenderLeda: public Render
{
 protected:
  window *W;
  array<list<polygon> > Env;
  array<list<polygon> > Bodies;
  bool FirstFrameDisplayed;
  vector PreviousFrame;
 public:
  int TransformDim; // Transformation dim: either 3 or 6

  RenderLeda();
  RenderLeda(string filepath);
  RenderLeda(Scene *s, string filepath);
  virtual ~RenderLeda();

  virtual void ShowCurrentAnimationFrame();

  virtual void Init();
  virtual void Reset();
  virtual void HandleEvents();
  virtual void Terminate();

  void InitData();
  void DrawBodies(const vector &x);
  void DrawEnv();

  virtual void DrawPath();
};


#include <LEDA/UNDEFINE_NAMES.h>

#endif
