//----------------------------------------------------------------------
//               The Motion Strategy Library (MSL)
//----------------------------------------------------------------------
//
// Copyright (c) 1998-2000 Iowa State University and Steve LaValle.  
// All Rights Reserved.
// 
// Permission to use, copy, and distribute this software and its 
// documentation is hereby granted free of charge, provided that 
// (1) it is not a component of a commercial product, and 
// (2) this notice appears in all copies of the software and
//     related documentation. 
// 
// Iowa State University and the author make no representations
// about the suitability or fitness of this software for any purpose.  
// It is provided "as is" without express or implied warranty.
//----------------------------------------------------------------------

#include <math.h>
#include <stdlib.h>

#include <LEDA/file.h>
#include <LEDA/array.h>
#include <LEDA/list.h>
#include <LEDA/matrix.h>
#include <LEDA/point.h>
#include <LEDA/polygon.h>
#include <LEDA/stream.h>
#include <LEDA/string.h>
#include <LEDA/vector.h>
#include <LEDA/graph.h>
#include <LEDA/d3_point.h>
#include <LEDA/window.h>
#include <LEDA/menu.h>

// Include all models
#include "model.h"
#include "modelmisc.h"
#include "model2d.h"
#include "model3d.h"

// Include all geometries
#include "geom.h"
#include "geomPQP.h"

#include "defs.h"

#include <LEDA/REDEFINE_NAMES.h>

void SetupProblem(Model *&m, Geom *&g, string path) {
  // Make them null initially
  m = NULL;
  g = NULL;

  // Models from modelmisc.h
  if (is_file(path+"Model1D"))
    m = new Model1D(path);
  if (is_file(path+"ModelLinear"))
    m = new ModelLinear(path);
  if (is_file(path+"ModelND"))
    m = new ModelND(path);
  if (is_file(path+"ModelNintegrator"))
    m = new ModelNintegrator(path);

  // Models from model2d.h
  if (is_file(path+"Model2DPoint"))
    m = new Model2DPoint(path);
  if (is_file(path+"Model2DPointCar"))
    m = new Model2DPointCar(path);
  if (is_file(path+"Model2DRigid"))
    m = new Model2DRigid(path);
  if (is_file(path+"Model2DRigidCar"))
    m = new Model2DRigidCar(path);
  if (is_file(path+"Model2DRigidCarForward"))
    m = new Model2DRigidCarForward(path);
  if (is_file(path+"Model2DRigidCarSmooth"))
    m = new Model2DRigidCarSmooth(path);
  if (is_file(path+"Model2DRigidCarSmoothTrailer"))
    m = new Model2DRigidCarSmoothTrailer(path);
  if (is_file(path+"Model2DRigidCarSmooth2Trailers"))
    m = new Model2DRigidCarSmooth2Trailers(path);
  if (is_file(path+"Model2DRigidCarSmooth3Trailers"))
    m = new Model2DRigidCarSmooth3Trailers(path);
  if (is_file(path+"Model2DRigidCarSmooth3Trailers"))
    m = new Model2DRigidCarSmooth3Trailers(path);
  if (is_file(path+"Model2DRigidDyncar"))
    m = new Model2DRigidDyncar(path);
  if (is_file(path+"Model2DRigidDyncarNtire"))
    m = new Model2DRigidDyncarNtire(path);
  if (is_file(path+"Model2DRigidMulti"))
    m = new Model2DRigidMulti(path);
  if (is_file(path+"Model2DRigidChain"))
    m = new Model2DRigidChain(path);

  // Models from model3d.h
  if (is_file(path+"Model3DRigid"))
    m = new Model3DRigid(path);
  if (is_file(path+"Model3DRigidMulti"))
    m = new Model3DRigidMulti(path);
  if (is_file(path+"Model3DRigidChain"))
    m = new Model3DRigidChain(path);
  if (is_file(path+"Model3DRigidTree"))
    m = new Model3DRigidTree(path);

  // Geoms from geom.h
  if (is_file(path+"GeomLedaPolygonal"))
    g = new GeomLedaPolygonal(path);
  if (is_file(path+"GeomLedaPolygonalRigid"))
    g = new GeomLedaPolygonalRigid(path);

  // Geoms from geomPQP.h
  if (is_file(path+"GeomPQP2DPoint"))
    g = new GeomPQP2DPoint(path);
  if (is_file(path+"GeomPQP2DRigid"))
    g = new GeomPQP2DRigid(path);
  if (is_file(path+"GeomPQP2DRigidMulti"))
    g = new GeomPQP2DRigidMulti(path);
  if (is_file(path+"GeomPQP3DRigid"))
    g = new GeomPQP3DRigid(path);
  if (is_file(path+"GeomPQP3DRigidMulti"))
    g = new GeomPQP3DRigidMulti(path);

  if (m == NULL) // Make a default Model
    m = new Model2DPoint(path);
  if (g == NULL) // Make a default Geom
    g = new GeomLedaPolygonal(path);

}



#include <LEDA/UNDEFINE_NAMES.h>
