//----------------------------------------------------------------------
//               The Motion Strategy Library (MSL)
//----------------------------------------------------------------------
//
// Copyright (c) 1998-2000 Iowa State University and Steve LaValle.  
// All Rights Reserved.
// 
// Permission to use, copy, and distribute this software and its 
// documentation is hereby granted free of charge, provided that 
// (1) it is not a component of a commercial product, and 
// (2) this notice appears in all copies of the software and
//     related documentation. 
// 
// Iowa State University and the author make no representations
// about the suitability or fitness of this software for any purpose.  
// It is provided "as is" without express or implied warranty.
//----------------------------------------------------------------------

#include <stdlib.h>
#include <stream.h>

#include <LEDA/list.h>
#include <LEDA/vector.h>
#include <LEDA/array2.h>

#define OUTERSTEPS 40
#define INNERSTEPS 20
#define XTRANS 0.0
#define YTRANS 15.0
#define ZTRANS 0.0
#define INNERRADIUS 4.0
#define OUTERRADIUS 15.0
#define PI 3.1415926535897932385


main(int argc, char **argv)
{ 
  list<vector> pl;
  vector p1,p2,p3;
  int i,j;
  double u,v;
  vector x(3);
  array2<vector> torus(OUTERSTEPS,INNERSTEPS);

  for (i = 0; i < OUTERSTEPS; i++) {
    u = i*2.0*PI/OUTERSTEPS;
    for (j = 0; j < INNERSTEPS; j++) {
      v = j*2.0*PI/INNERSTEPS;
      x[0] = XTRANS + (OUTERRADIUS + INNERRADIUS * cos(v))*cos(u);
      x[1] = YTRANS + INNERRADIUS * sin(v);
      x[2] = ZTRANS + (OUTERRADIUS + INNERRADIUS * cos(v))*sin(u);
      torus(i,j) = x;
    }
  }

  for (i = 0; i < OUTERSTEPS; i++) {
    for (j = 0; j < INNERSTEPS; j++) {
      p1 = torus(i,j);
      p2 = torus((i+1) % OUTERSTEPS,j);
      p3 = torus(i,(j+1) % INNERSTEPS);
      cout << "(" << p1[0] << "," << p1[1] << "," << p1[2] << ") ";
      cout << "(" << p2[0] << "," << p2[1] << "," << p2[2] << ") ";
      cout << "(" << p3[0] << "," << p3[1] << "," << p3[2] << ")\n";
      p1 = torus(i,(j+1) % INNERSTEPS);
      p2 = torus((i+1) % OUTERSTEPS,j);
      p3 = torus((i+1) % OUTERSTEPS ,(j+1) % INNERSTEPS);
      cout << "(" << p1[0] << "," << p1[1] << "," << p1[2] << ") ";
      cout << "(" << p2[0] << "," << p2[1] << "," << p2[2] << ") ";
      cout << "(" << p3[0] << "," << p3[1] << "," << p3[2] << ")\n";
    }
  }

}
  

